# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.27.0
    Generated by: https://openapi-generator.tech
"""

from gentrace.paths.v2_datasets.get import V2DatasetsGet
from gentrace.paths.v2_datasets_id.get import V2DatasetsIdGet
from gentrace.paths.v2_datasets_id.post import V2DatasetsIdPost
from gentrace.paths.v2_datasets.post import V2DatasetsPost
from gentrace.paths.v2_evaluations_bulk.post import V2EvaluationsBulkPost
from gentrace.paths.v2_evaluations.get import V2EvaluationsGet
from gentrace.paths.v2_evaluators.get import V2EvaluatorsGet
from gentrace.paths.v2_feedback_id.get import V2FeedbackIdGet
from gentrace.paths.v2_feedback_id.patch import V2FeedbackIdPatch
from gentrace.paths.v2_feedback.post import V2FeedbackPost
from gentrace.paths.v2_folders.get import V2FoldersGet
from gentrace.paths.v2_folders_id.get import V2FoldersIdGet
from gentrace.paths.v2_pipelines.get import V2PipelinesGet
from gentrace.paths.v2_runs_id.get import V2RunsIdGet
from gentrace.paths.v2_test_cases.get import V2TestCasesGet
from gentrace.paths.v2_test_cases_id.delete import V2TestCasesIdDelete
from gentrace.paths.v2_test_cases_id.get import V2TestCasesIdGet
from gentrace.paths.v2_test_results.get import V2TestResultsGet


class V2Api(
    V2DatasetsGet,
    V2DatasetsIdGet,
    V2DatasetsIdPost,
    V2DatasetsPost,
    V2EvaluationsBulkPost,
    V2EvaluationsGet,
    V2EvaluatorsGet,
    V2FeedbackIdGet,
    V2FeedbackIdPatch,
    V2FeedbackPost,
    V2FoldersGet,
    V2FoldersIdGet,
    V2PipelinesGet,
    V2RunsIdGet,
    V2TestCasesGet,
    V2TestCasesIdDelete,
    V2TestCasesIdGet,
    V2TestResultsGet,
):
    """NOTE: This class is auto generated by OpenAPI Generator
    Ref: https://openapi-generator.tech

    Do not edit the class manually.
    """
    pass
