# coding: utf-8

"""
    Gentrace API

    These API routes are designed to ingest events from clients.  # noqa: E501

    The version of the OpenAPI document: 0.27.0
    Generated by: https://openapi-generator.tech
"""

from datetime import date, datetime  # noqa: F401
import decimal  # noqa: F401
import functools  # noqa: F401
import io  # noqa: F401
import re  # noqa: F401
import typing  # noqa: F401
import typing_extensions  # noqa: F401
import uuid  # noqa: F401

import frozendict  # noqa: F401

from gentrace import schemas  # noqa: F401


class UnixSecondsNullable(
    schemas.Float32Base,
    schemas.NumberBase,
    schemas.NoneBase,
    schemas.Schema,
    schemas.NoneDecimalMixin
):
    """NOTE: This class is auto generated by OpenAPI Generator.
    Ref: https://openapi-generator.tech

    Do not edit the class manually.

    Timestamp in seconds since the UNIX epoch. Can be transformed into a Date object.
    """


    class MetaOapg:
        format = 'float'


    def __new__(
        cls,
        *_args: typing.Union[None, decimal.Decimal, int, float, ],
        _configuration: typing.Optional[schemas.Configuration] = None,
    ) -> 'UnixSecondsNullable':
        return super().__new__(
            cls,
            *_args,
            _configuration=_configuration,
        )
